SPOOL triggers.log
/*******************************************************************************************
  TRIGGERS PER CALCULAR VALORS DE CONSUM DESPRES DELS INSERTS
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_aft_ins_lectura_comptador
   AFTER INSERT OR UPDATE
   ON lectura_comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_AFT_INS_LECTURA_COMPTADOR
    PROPSIT:   Trigger que afegeix de forma automtica els valors dins els camps
                "consum_actual" i "consum_total" de la taula COMPTADOR.
                - Consum actual: Consum del darrer mes informat a LECTURA_COMPTADOR
                - Consum total: sumatori de tots els consums informats a LECTURA_COMPTADOR
*******************************************************************************************/
DECLARE
   v_cons_act       NUMBER;
   v_cons_tot       NUMBER;
   v_data_lectura   DATE;
BEGIN
   -- Agafarem el consum total amb la suma dels consums mensuals
   BEGIN
      SELECT NVL (consum_actual, 0), NVL (consum_total, 0), NVL (darrera_lectura, '01/01/1900')
        INTO v_cons_act, v_cons_tot, v_data_lectura
        FROM comptador
       WHERE idcontracte = :NEW.idcontracte;

      -- Agafarem el valor mes alt de lectura per la data i el consum actual
      IF :NEW.data_lectura > v_data_lectura
      THEN
         v_data_lectura := :NEW.data_lectura;
         v_cons_act := :NEW.valor_lectura;
      END IF;

      -- Sumarem el valor total
      v_cons_tot := v_cons_tot + :NEW.valor_lectura;

      UPDATE comptador
         SET consum_actual = v_cons_act,
             consum_total = v_cons_tot,
             darrera_lectura = v_data_lectura
       WHERE idcontracte = :NEW.idcontracte;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         UPDATE comptador
            SET consum_actual = :NEW.valor_lectura,
                consum_total = :NEW.valor_lectura,
                darrera_lectura = :NEW.data_lectura
          WHERE idcontracte = :NEW.idcontracte;
   END;
END;
/

CREATE OR REPLACE TRIGGER tg_aft_ins_hist_central_prod
   AFTER INSERT OR UPDATE
   ON hist_central_prod
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_AFT_INS_HIST_CENTRAL_PROD
    PROPSIT:   Trigger que afegeix de forma automtica dins la taula CENTRAL_PRODUCCIO
                el valor d'energia ms alt que tenim a la taula HIST_CENTRAL_PROD.
*******************************************************************************************/
DECLARE
   v_ener_max   NUMBER;
BEGIN
   -- Agafarem el valor mes alt que hem obtingut de l'histric
   BEGIN
      SELECT NVL (energia_max, 0)
        INTO v_ener_max
        FROM central_produccio
       WHERE idcentralprod = :NEW.idcentralprod;

      IF :NEW.energia_mes > v_ener_max
      THEN
         UPDATE central_produccio
            SET energia_max = :NEW.energia_mes
          WHERE idcentralprod = :NEW.idcentralprod;
      END IF;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         UPDATE central_produccio
            SET energia_max = :NEW.energia_mes
          WHERE idcentralprod = :NEW.idcentralprod;
   END;
END;
/

/********************************** MDUL ESTADSTIC **************************************/

/*******************************************************************************************
    estadstic 1
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_1_ins_lc
   AFTER INSERT
   ON lectura_comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_1_INS_LC
    PROPSIT:   Trigger del mdul estadstic, afegeix dins la taula d'estadstic
                EST_CONSUMCOMPTPROD el consum que hem inserit a la taula LECTURA_COMPTADOR
                - Comprovarem si existeix l'id de central de producci
                - Si existeix, sumarem el camp valor_lectura al camp consum
                - Si no existeix, afegirem el registre amb l'id de la central de producci
                  i el camp valor_lectura.
*******************************************************************************************/
DECLARE
   v_exists          NUMBER;
   v_idcentralprod   NUMBER;
BEGIN
   BEGIN
      -- Agafarem el id de la central de producci
      SELECT cd.idcentralprod
        INTO v_idcentralprod
        FROM distribucio_comptador dc, central_distribucio cd
       WHERE     dc.idcontracte = :NEW.idcontracte
             AND cd.idcentraldis = dc.idcentraldis
             AND ROWNUM = 1;

      -- Comprovarem si aquest id de central de producci ja est donat d'alta a la taula
      -- d'estadstica
      SELECT '1'
        INTO v_exists
        FROM est_consumcomptprod
       WHERE idcentralprod = v_idcentralprod;

      -- En cas de que ja existeixi, actualitzarem la dada del consum, sumant la xifra
      -- que hem inserit a la taula LECTURA_COMPTADOR
      UPDATE est_consumcomptprod
         SET consum_total = consum_total + :NEW.valor_lectura
       WHERE idcentralprod = v_idcentralprod;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         -- Si no existia, farem un insert a la taula estadstica
         INSERT INTO est_consumcomptprod (idcentralprod, consum_total)
              VALUES (v_idcentralprod, :NEW.valor_lectura);
   END;
END;
/

/*******************************************************************************************
    estadstic 2
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_2_ins_lc
   AFTER INSERT
   ON lectura_comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_2_INS_LC
    PROPSIT:   Trigger del mdul estadstic, afegeix dins la taula d'estadstic
                EST_CONSUMLINIAANY el consum que hem inserit a la taula LECTURA_COMPTADOR.
                Afegirem els camps d'idlinia, any, consum_total, numero de contractes
                associats a la lnia i mitja de consum de la lnia.
                - Comprovarem que existeix el registre per les lnies i l'any agafat
                - En cas d'existir, actualitzarem el consum de les lnies de comunicaci
                - En cas de no existir, afegirem un nou registre amb l'id de la lnia,
                  l'any que va computar el consum i el valor del consum
*******************************************************************************************/
DECLARE
   v_exists       NUMBER;
   v_idlinia      NUMBER;
   v_any          NUMBER;
   v_contractes   NUMBER;
   v_consum       NUMBER;
   v_mitja        NUMBER;

   CURSOR c_lnies (
      p_idcontracte NUMBER)
   IS
      -- guardar id i capacitat max de la lnia de comunicaci
      SELECT l.idlinia, l.capacitat_max
        FROM distribucio_comptador dc, central_distribucio cd, linia_comunicacio l
       WHERE     dc.idcontracte = p_idcontracte
             AND cd.idcentraldis = dc.idcentraldis
             AND l.idlinia = cd.idlinia;
BEGIN
   -- any on afegim el registre
   v_any := TO_NUMBER (TO_CHAR (:NEW.data_inici, 'YYYY'));

   FOR reg IN c_lnies (:NEW.idcontracte)
   LOOP
      -- Controlarem si tenim el registre dins la taula d'estadstica
      -- primer agafarem el nombre de contractes que tenim amb la lnia seleccionada
      SELECT COUNT (DISTINCT (idcontracte))
        INTO v_contractes
        FROM distribucio_comptador dc, central_distribucio cd, linia_comunicacio l
       WHERE     cd.idcentraldis = dc.idcentraldis
             AND l.idlinia = cd.idlinia
             AND l.idlinia = reg.idlinia;

      -- Controlarem l'existncia del registre dins la taula estadstica
      BEGIN
         SELECT '1'
           INTO v_exists
           FROM est_consumliniaany
          WHERE idlinia = reg.idlinia AND p_any = v_any;

         -- Existeix el registre, per tant, farem els calculs necesaris per obtenir
         -- la nova mitja
         -- Agafarem el total del consum de l'any de la linia
         SELECT consum
           INTO v_consum
           FROM est_consumliniaany
          WHERE idlinia = reg.idlinia AND p_any = v_any;

         -- Sumarem la part que s'ha donat d'alta
         v_consum := v_consum + :NEW.valor_lectura;
         -- Ara calcularem la mitja (consum total/contractes)
         v_mitja := round(v_consum / v_contractes, 2);

         -- Afegirem les noves dades a la taula
         UPDATE est_consumliniaany
            SET consum = v_consum, num_compt = v_contractes, mitja_consum = v_mitja
          WHERE idlinia = reg.idlinia AND p_any = v_any;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            -- En cas de no existir el registre, agafarem el consum inicial i el nombre
            -- de comptadors per fer la mitja
            -- Sumarem la part que s'ha donat d'alta
            v_consum := :NEW.valor_lectura;
            -- Ara calcularem la mitja (consum total / ncontractes)
            v_mitja := round(v_consum / v_contractes, 2);

            -- crearem el registre dins la taula
            INSERT INTO est_consumliniaany (idlinia,
                                            p_any,
                                            consum,
                                            num_compt,
                                            mitja_consum)
                 VALUES (reg.idlinia,
                         v_any,
                         v_consum,
                         v_contractes,
                         v_mitja);
      END;
   END LOOP;
END;
/

/*******************************************************************************************
    estadstic 3
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_3_ins_lc
   AFTER INSERT
   ON lectura_comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_3_INS_LC
    PROPSIT:   Trigger del mdul estadstic, afegeix dins la taula d'estadstic
                EST_LINIAMESCARREGADA el registre amb l'idlinia que est ms carregada.
                Anirem comprovant cada vegada que actualitzem el valor del comptador
                CONSUM_TOTAL i comprovarem si realment s la idlinia ms carregada.
                - Comprovarem el nou consum de la lnia que s'ha actualitzat
                - Compararem el valor amb la lnia ms carregada actualment
                - En cas de que sigui un percentatge superior, canviarem el registre,
                  guardant la nova lnia com lnia ms carregada.
*******************************************************************************************/
DECLARE
   v_exists         NUMBER;
   v_idlinia        NUMBER;
   v_any            NUMBER;
   v_contractes     NUMBER;
   v_consum_total   NUMBER;
   v_perc_tab       NUMBER;
   v_perc_consum    NUMBER;

   CURSOR c_lnies (
      p_idcontracte NUMBER)
   IS
      -- Agafarem el id de la central de producci
      SELECT l.idlinia, l.capacitat_max
        FROM distribucio_comptador dc, central_distribucio cd, linia_comunicacio l
       WHERE     dc.idcontracte = p_idcontracte
             AND cd.idcentraldis = dc.idcentraldis
             AND l.idlinia = cd.idlinia;
BEGIN
   -- Agafarem les lnies connectades al comptador modificat
   -- Com podem tenir ms d'una central de distribuci connectada amb una lnia diferent
   -- i agafarem el mateix valor per a cada una de les lnies conectades la comptador,
   -- revisarem el percentatge d'energia respecte a la capacitat de la lnia.
   FOR reg IN c_lnies (:NEW.idcontracte)
   LOOP
      -- Cercant el percentatge de consum, comprovarem tamb si hi ha registre a la taula
      -- d'estadstica
      -- Primer obtenim el consum total de la linia
      SELECT NVL (SUM (c.consum_actual), 0)
        INTO v_consum_total
        FROM comptador c,
             distribucio_comptador dc,
             central_distribucio cd,
             linia_comunicacio l
       WHERE     l.idlinia = reg.idlinia
             AND cd.idlinia = l.idlinia
             AND dc.idcentraldis = cd.idcentraldis
             AND c.idcontracte = dc.idcontracte;

      -- Agafarem el percentatge d'energia consumida per la lnia.
      v_perc_consum := (v_consum_total / reg.capacitat_max) * 100;

      BEGIN
         SELECT perc_carrega
           INTO v_perc_tab
           FROM est_liniamescarregada
          WHERE ROWNUM = 1;

         -- Existeix el registre, per tant, comprovarem si tenim una lnia amb un percentatge
         -- de crrega major a l'actual
         -- Comprovam si el nou percentatge s major que el percentatge que teniem
         -- guardat anteriorment
         IF v_perc_consum > v_perc_tab
         THEN
            -- tenim un valor major amb l'actualitzaci de LECTURA_COMPTADOR, per tant,
            -- actualitzarem la taula estadstica.
            UPDATE est_liniamescarregada
               SET idlinia = reg.idlinia,
                   carrega_total = v_consum_total,
                   capacitat_max = reg.capacitat_max,
                   perc_carrega = v_perc_consum
             WHERE ROWNUM = 1;
         END IF;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            -- En cas de no existir el registre, ficarem la priemra lnia que hem calculat
            -- crearem el registre dins la taula
            INSERT INTO est_liniamescarregada (idlinia,
                                               carrega_total,
                                               capacitat_max,
                                               perc_carrega)
                 VALUES (reg.idlinia,
                         v_consum_total,
                         reg.capacitat_max,
                         v_perc_consum);
      END;
   END LOOP;
END;
/

/*******************************************************************************************
    estadstic 4
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_4_ins_lcom
   AFTER INSERT
   ON linia_comunicacio
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_4_INS_LCOM
    PROPSIT:   Trigger del mdul estadstic, si donam d'alta una nova lnia de comunicaci,
                informarem d'aquesta alta al totalitzador de la taula estadstica.
                EST_ANYLINIESENCONSUM el consum que hem inserit a la taula LECTURA_COMPTADOR.
                Aquest trigger inicialitza els registres de cada any, comenant pel 2001,
                per tenir la taula pendent d'actualitzar quan s'insereixi un nou consum.
                Com que noms estarem donant d'alta la nova lnia, encara no tindr consum
                assignat, per tant, aquest trigger noms comptabilitza la lnia al
                totalitzador de lnies.
                - per a cada any (2001 fins 2012): comprovem que tenim el registre de l'any
                  creat.
                - Si ja el tenim creat, sumarem 1 al totalitzador, i calcularem el nou
                  percentatge.
*******************************************************************************************/
DECLARE
   v_exists        NUMBER;
   v_numlinies     NUMBER;
   v_superen       NUMBER;
   v_perc_linies   NUMBER;
   v_consum        NUMBER;

   CURSOR c_linies (pc_any NUMBER)
   IS
        SELECT p_any,
               num_linies,
               superen,
               perc_linies
          FROM est_anyliniesenconsum
         WHERE p_any = pc_any
      ORDER BY p_any;
BEGIN
   -- Per defecte, crearem les lnies desde 2001 fins 2012
   FOR cont IN 2001 .. 2012
   LOOP
      BEGIN
         -- Controlarem que hi hagi al menys un registre per anar sumant els totals
         -- de les lnies
         SELECT '1'
           INTO v_exists
           FROM est_anyliniesenconsum
          WHERE p_any = cont AND ROWNUM = 1;

         -- en cas d'existir, actualitzarem totes les lnies existents
         FOR reg IN c_linies (cont)
         LOOP
            -- noms actualitzarem el totalitzador de lnies, ja que com encara no te lectures,
            -- no superar el 50% de lnia consumida.
            v_numlinies := reg.num_linies + 1;
            -- Calcularem el nou percentatge
            v_perc_linies := (reg.superen / v_numlinies) * 100;

            -- Actualitzarem tots els registres per sumar la nova linia
            UPDATE est_anyliniesenconsum
               SET num_linies = v_numlinies, perc_linies = v_perc_linies
             WHERE p_any = reg.p_any;
         END LOOP;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            -- En cas de no existir el registre, no tocarem la taula estadstica, deixarem
            -- que la carregui el trigger de LECTURA_COMPTADOR
            INSERT INTO est_anyliniesenconsum (p_any,
                                               num_linies,
                                               superen,
                                               perc_linies)
                 VALUES (cont,
                         1,
                         0,
                         0);
      END;
   END LOOP;
END;
/

CREATE OR REPLACE TRIGGER tg_estadstic_4_ins_est_c
   AFTER INSERT OR UPDATE
   ON est_consumliniaany
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_4_INS_EST_C
    PROPSIT:   Trigger del mdul estadstic, aprofitant una altra taula estadstica,
                EST_CONSUMLINIAANY, on tenim el control de consum de les lnies per any,
                comprovem la quantitat de lnies que superen el 50% d'energia consumida.
                Amb el trigger que tenim quan fem l'alta de lnia de comunicaci,
                actualitzem el totalitzador, amb aquest trigger, aprofitarem aquest valor
                de totalitzador per calcular el percentatge de lnies que superen el
                50% de l'energia consumida.
                - Comprovarem la lnia actualitzada a la taula EST_CONSUMLINIAANY
                - Calcularem si supera el 50% d'energia consumida per l'any que s'ha
                  actualitzat.
                - En cas de superar el 50%, sumarem 1 al camp "superen", que, juntament
                  amb el totalitzador, ens servir per treure el percentatge de lnies
                  que superen el 50% d'energia consumida per l'any informat.
*******************************************************************************************/
DECLARE
   v_consum_ant        NUMBER;
   v_consum_act        NUMBER;
   v_perc_energia      NUMBER;
   v_numlinies         NUMBER;
   v_superen           NUMBER;
   v_perc_linies       NUMBER;
   v_capacitat_linia   NUMBER;
BEGIN
   -- Inicialitzam el valor de lnies que superen el 50% d'energia consumida
   -- Controlarem el valor anterior per veure si ja estava donat d'alta
   v_consum_ant := :OLD.consum;
   v_consum_act := :NEW.consum;

   SELECT num_linies, superen, perc_linies
     INTO v_numlinies, v_superen, v_perc_linies
     FROM est_anyliniesenconsum
    WHERE p_any = :NEW.p_any;

   SELECT capacitat_max
     INTO v_capacitat_linia
     FROM linia_comunicacio
    WHERE idlinia = :NEW.idlinia;

   v_perc_energia := (v_consum_ant / (v_capacitat_linia * 12)) * 100;

   IF v_perc_energia <= 50
   THEN
      -- Si no tenia un percentatge superior al 50% comprovarem si el t ara, per afegir-lo
      -- al sumatori
      v_perc_energia := (v_consum_act / (v_capacitat_linia * 12)) * 100;

      IF v_perc_energia > 50
      THEN
         v_superen := v_superen + 1;
         v_perc_linies := (v_superen / v_numlinies) * 100;

         -- El canvi fa que entri en el sumatori
         UPDATE est_anyliniesenconsum
            SET superen = v_superen, perc_linies = v_perc_linies
          WHERE p_any = :NEW.p_any;
      END IF;
   END IF;
END;
/

/*******************************************************************************************
    estadstic 5
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_5_ins_lc
   AFTER INSERT
   ON lectura_comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_5_INS_LC
    PROPSIT:   Trigger del mdul estadstic, afegeix dins la taula d'estadstic
                EST_CONSUMCPANY l'energia emesa per la central de producci en l'any
                que hem definit com a parmetre en la taula LECTURA_COMPTADOR.
                - Agafarem l'id de la central de producci i l'any que se suma el valor de
                  la lectura del comptador.
                - Sumarem el valor a la taula EST_CONSUMCPANY amb el filtre de l'id de
                  central de producci i l'any.
                - Si no existeix cap registre a la taula EST_CONSUMCPANY, per aquest any
                  i aquest id de central de producci, farem els segents clculs:
                    - En cas de que l'energia consumida sigui inferior al 30% de l'energia
                      emesa, sumarem 1 al valor de la taula EST_SUMACPANY.
                    - Si s igual o superior al 30%, no sumarem el valor.
*******************************************************************************************/
DECLARE
   v_exists          NUMBER;
   v_idcentralprod   NUMBER;
   v_energia_any     NUMBER;
   v_any             NUMBER;
   v_suma_cp         NUMBER;
   v_consum          NUMBER;
   v_perc            NUMBER;
BEGIN
   -- primer agafarem l'any on afegim el registre
   v_any := TO_NUMBER (TO_CHAR (:NEW.data_inici, 'YYYY'));

   -- Agafarem l'id de la central de producci a la que pertanya el contracte
   SELECT cd.idcentralprod
     INTO v_idcentralprod
     FROM distribucio_comptador dc, central_distribucio cd
    WHERE dc.idcontracte = :NEW.idcontracte AND cd.idcentraldis = dc.idcentraldis AND ROWNUM = 1;

   -- Guardam la suma de l'energia emesa per la central de producci en l'any que tenim
   SELECT SUM (energia_mes)
     INTO v_energia_any
     FROM hist_central_prod
    WHERE idcentralprod = v_idcentralprod AND TO_CHAR (data_inici, 'YYYY') = v_any;

   -- Controlarem l'existncia del registre dins la taula estadstica
   BEGIN
      SELECT consum, perc_consum
        INTO v_consum, v_perc
        FROM est_consumcpany
       WHERE idcentralprod = v_idcentralprod AND p_any = v_any;

      -- Existeix el registre
      -- Calcularem el nou percentatge
      v_consum := v_consum + :NEW.valor_lectura;
      v_perc := (v_consum / v_energia_any) * 100;

      -- Sumarem el consum introduit a LECTURA_COMPTADOR
      UPDATE est_consumcpany
         SET energia_any = v_energia_any, consum = v_consum, perc_consum = v_perc
       WHERE idcentralprod = v_idcentralprod AND p_any = v_any;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         -- En cas de no existir el registre, agafarem el consum inicial i
         -- crearem el registre dins la taula
         v_perc := (:NEW.valor_lectura / v_energia_any) * 100;

         INSERT INTO est_consumcpany (idcentralprod,
                                      p_any,
                                      energia_any,
                                      consum,
                                      perc_consum)
              VALUES (v_idcentralprod,
                      v_any,
                      v_energia_any,
                      :NEW.valor_lectura,
                      v_perc);

         -- Comprovaci: existeix registre a la taula EST_SUMACPANY
         BEGIN
            SELECT total
              INTO v_suma_cp
              FROM est_sumacpany
             WHERE p_any = v_any;

            -- Existeix -> Comprovaci de percentatge
            IF v_perc < 30
            THEN
               -- v_perc < 30 = TRUE -> suma 1 al total
               UPDATE est_sumacpany
                  SET total = total + 1
                WHERE p_any = v_any;
            END IF;
         EXCEPTION
            -- No Existeix -> Comprovaci de percentatge:
            -- si el percentatge s menor de 30 = sumarem 1 central de producci
            -- si el percentatge s major o igual a 30 = deixarem el camp del total a 0
            WHEN NO_DATA_FOUND
            THEN
               IF v_perc < 30
               THEN
                  v_suma_cp := 1;
               ELSE
                  v_suma_cp := 0;
               END IF;

               -- Afegirem el nou registre
               INSERT INTO est_sumacpany (p_any, total)
                    VALUES (v_any, v_suma_cp);
         END;
   END;
END;
/

CREATE OR REPLACE TRIGGER tg_estadstic_5_ins_est_cp
   AFTER UPDATE
   ON est_consumcpany
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_5_INS_EST_CP
    PROPSIT:   Trigger del mdul estadstic, una vegada actualitzada la taula estadstica
                del consum anual de les centrals de producci, afegirem aquelles centrals
                que tinguin un consum inferior del 30% de l'energia emesa, en el sumatori
                de la taula EST_CONSUMCPANY.
                - Comprovarem el percentatge anterior i l'actual.
                - En cas de que el percentatge anterior a l'actualitzaci sigui inferior al 30%,
                  contemplarem dues opcions:
                    - Si ara s igual o superior al 30%, restarem 1 al sumatori.
                    - En cas de que encara sigui inferior, no farem res.
                - Amb el cas contrari, que el percentatge anterior sigui igual o superior al
                  30%, tamb contemplarem 2 casos:
                    - Si ara s menor al 30%, sumarem 1 al sumatori.
                    - En cas de que encara sigui igual o superior al 30%, no farem res.
*******************************************************************************************/
DECLARE
   v_perc_ant   NUMBER;
   v_perc_act   NUMBER;
BEGIN
   -- Controlarem el valor anterior per veure si ja tenia un 30% de produccio
   v_perc_ant := :OLD.perc_consum;
   v_perc_act := :NEW.perc_consum;

   -- Comprovarem si ha hagut canvis en el percentatge de consum
   IF v_perc_ant < 30
   THEN
      IF v_perc_act >= 30
      THEN
         -- Ha canviat el valor de la condici, per tant, restarem 1 al sumatori
         UPDATE est_sumacpany
            SET total = total - 1
          WHERE p_any = :NEW.p_any;
      END IF;
   ELSE
      IF v_perc_act < 30
      THEN
         -- Ha canviat el valor de la condici, en aquest cas, sumarem 1 al registre
         UPDATE est_sumacpany
            SET total = total + 1
          WHERE p_any = :NEW.p_any;
      END IF;
   END IF;
END;
/

/*******************************************************************************************
    estadstic 6
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_6_upd_c
   AFTER UPDATE
   ON comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_6_UPD_C
    PROPSIT:   Trigger del mdul estadstic, afegeix a la taula EST_TOP10COMPTADORS els
                comptadors que han tingut ms consum histricament.
                - Comprovem primer que el contracte modificat no estigui ja ficat dins
                  la taula.
                - Si ja existeix al top10, s'actualitza el registre del contracte modificat.
                - En cas de no existir, farem dues comprovacions.
                    - La taula no t encara 10 registres: afegim el nou registre.
                    - La taula ja t 10 registres: comprovem si la nova actualitzaci supera
                      en consum al darrer consum de la taula top10, en cas de superar-lo,
                      eliminarem el darrer registre i ficarem el que acabem d'actualitzar.
                      En cas contrari, no farem res, deixant la taula top 10 com estava.
*******************************************************************************************/
DECLARE
   v_exists          NUMBER;
   v_consum_total    NUMBER;
   v_consum_min      NUMBER;
   v_contracte_min   NUMBER;
   v_rownum          NUMBER;
BEGIN
   -- Commprovem si existeix el contracte modificat a la taula top10comptadors
   SELECT '1'
     INTO v_exists
     FROM est_top10comptadors
    WHERE idcontracte = :NEW.idcontracte;

   -- si existeix, actualitzarem el registre
   UPDATE est_top10comptadors
      SET consum_total = :NEW.consum_total
    WHERE idcontracte = :NEW.idcontracte;
EXCEPTION
   -- Si no existeix el registre, comprovarem que hi ha menys de 10 registres o que el consum
   -- total s major que el menor de tots els consums.
   WHEN NO_DATA_FOUND
   THEN
      SELECT NVL (MAX (ROWNUM), 0) INTO v_rownum FROM est_top10comptadors;

      IF v_rownum < 10
      THEN
         -- podem fer l'insert sense problemes
         INSERT INTO est_top10comptadors (idcontracte, consum_total)
              VALUES (:NEW.idcontracte, :NEW.consum_total);
      ELSE
           -- Ja tenim el top 10, comprovarem si se pot eliminar un registre per afegir aquest
           -- nou registre
           SELECT MIN (idcontracte), consum_total
             INTO v_contracte_min, v_consum_min
             FROM est_top10comptadors
            WHERE consum_total = (SELECT MIN (consum_total) FROM est_top10comptadors)
         GROUP BY consum_total;

         -- Revisarem si el nou registre es superior al menor registre de la taula
         IF v_contracte_min < :NEW.consum_total
         THEN
            -- En cas que el nou registre sigui superior, farem la substituci a la taula
            DELETE FROM est_top10comptadors
                  WHERE idcontracte = v_contracte_min;

            -- Insertarem el nou registre a la taula
            INSERT INTO est_top10comptadors (idcontracte, consum_total)
                 VALUES (:NEW.idcontracte, :NEW.consum_total);
         END IF;
      END IF;
END;
/

/*******************************************************************************************
    estadstic 7
*******************************************************************************************/

CREATE OR REPLACE TRIGGER tg_estadstic_7_ins_cli
   AFTER INSERT
   ON client
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_7_INS_CLI
    PROPSIT:   Trigger del mdul estadstic, suma el client donat d'alta al totalitzador
                de clients de la taula EST_CONSUMMIGCLIENTS.
                - Comprovarem que existeix el registre de consum mig de clients.
                - En cas d'existir, sumarem 1 al totalitzador de clients i tornarem a
                  calcular el consum mig.
                - En cas de no existir, inserirem un nou registre a la taula
                  EST_CONSUMMIGCLIENTS, amb els valors segents:
                  -  num_clients = 1 (el mateix client que s'ha donat d'alta)
                  -  consum_total = 0 (no t encara cap registre de consum,
                                       per tant, el seu valor s 0)
                  -  consum_mig = 0 (com noms tenim un client sense consum,
                                     el seu valor es 0)
*******************************************************************************************/
DECLARE
   v_consum_total   NUMBER;
   v_consum_mig     NUMBER;
   v_num_clients    NUMBER;
BEGIN
   --
   SELECT consum_mig, consum_total, num_clients
     INTO v_consum_mig, v_consum_total, v_num_clients
     FROM est_consummigclients;

   v_num_clients := v_num_clients + 1;
   v_consum_mig := round(v_consum_total / v_num_clients, 2);

   UPDATE est_consummigclients
      SET num_clients = v_num_clients, consum_mig = v_consum_mig;
EXCEPTION
   -- si no troba dades, inicialitzarem la taula de consum mig de clients.
   WHEN NO_DATA_FOUND
   THEN
      INSERT INTO est_consummigclients (num_clients, consum_total, consum_mig)
           VALUES (1, 0, 0);
END;
/

CREATE OR REPLACE TRIGGER tg_estadstic_7_ins_lc
   AFTER INSERT
   ON lectura_comptador
   FOR EACH ROW
/*******************************************************************************************
    NOM:        TG_estadstic_7_UPD_C
    PROPSIT:   Trigger del mdul estadstic, en actualitzar el consum d'un comptador,
                se calcula el consum mig de tots els clients.
                - Seleccionarem els valors actuals de la taula EST_CONSUMMIGCLIENTS.
                - Sumarem el nou valor dins el consum total.
                - Calcularem el consum mig amb les noves dades i actualitzarem el registre.
*******************************************************************************************/
DECLARE
   v_consum_total   NUMBER;
   v_consum_mig     NUMBER;
   v_num_clients    NUMBER;
BEGIN
   SELECT consum_mig, consum_total, num_clients
     INTO v_consum_mig, v_consum_total, v_num_clients
     FROM est_consummigclients;

   v_consum_total := v_consum_total + :NEW.valor_lectura;
   v_consum_mig := round(v_consum_total / v_num_clients, 2);

   UPDATE est_consummigclients
      SET consum_total = v_consum_total, consum_mig = v_consum_mig;
END;
/

SPOOL OFF;