SPOOL procediments.log
/

/******************** Llista comptadors consum mensual > 80% mitja ************************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc1 AS OBJECT (
   idcontracte   NUMBER,
   potenciamax   NUMBER,
   perc_consum   NUMBER
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc1 IS TABLE OF obj_proc1;
/

CREATE OR REPLACE FUNCTION f_consum_superior_80 (p_nomlocalitat IN VARCHAR2, p_data IN DATE)
   RETURN tab_proc1
IS
/*******************************************************************************************
    NOM:        f_consum_superior_80
    PROPSIT:   Funci que retorna una llista amb els comptadors que han superat el 80% del
                consum mitj de tots els comptadors de la ciutat en un perode concret.
                Valors de la llista:
                - idcontracte
                - Potenciamax
                - % consum_mes
    PARMETRES:
        IN:     p_nomlocalitat: Nom de la ciutat on pertanyen els comptadors
                p_data: Data que pertanya a un rang per comprovar el consum del mes
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna una llista amb els contractes, potencies i % de consum respecte a la mitja
            RSP = 'OK - f_consum_superior_80'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_consum_superior_80('MALLORCA', '10/10/2011'));
*******************************************************************************************/
   l_proc1    tab_proc1       := tab_proc1 ();
   err_null   EXCEPTION;
   v_cont     NUMBER          := 0;
   v_mitja    NUMBER;
   v_limit    NUMBER;
   v_consum   NUMBER;
   rsp        VARCHAR2 (2000);
   v_parin    VARCHAR2 (300);
   err_num    NUMBER;
   err_msg    VARCHAR2 (100);

   CURSOR c_consum_mensual (pc_nomlocalitat VARCHAR2, pc_data DATE, pc_mitja NUMBER)
   IS
      SELECT   c.idcontracte, c.potencia, lc.valor_lectura
          FROM lectura_comptador lc, comptador c, localitat l
         WHERE l.nomlocalitat = pc_nomlocalitat
           AND c.idlocalitat = l.idlocalitat
           AND lc.idcontracte = c.idcontracte
           AND lc.data_inici <= pc_data
           AND lc.data_fi >= pc_data
           AND lc.valor_lectura > pc_mitja
      ORDER BY lc.valor_lectura DESC;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   v_parin := 'p_nomlocalitat: ' || p_nomlocalitat || ', p_data: ' || p_data;

   -- Verifica que estan informats els parmetres d'entrada
   IF (TRIM (p_nomlocalitat) IS NULL OR TRIM (p_data) IS NULL)
   THEN
      RAISE err_null;
   END IF;

   -- Agafarem el valor de la mitja
   SELECT AVG (lc.valor_lectura)
     INTO v_mitja
     FROM lectura_comptador lc, comptador c, localitat l
    WHERE l.nomlocalitat = p_nomlocalitat
      AND c.idlocalitat = l.idlocalitat
      AND lc.idcontracte = c.idcontracte
      AND lc.data_inici <= p_data
      AND lc.data_fi >= p_data;

   -- Calcularem el valor corresponent al 80% del valor mitj
   v_limit := v_mitja * 1.80;

   FOR reg IN c_consum_mensual (p_nomlocalitat, p_data, v_limit)
   LOOP
      l_proc1.EXTEND;
      v_cont := v_cont + 1;
      -- Calcularem el % que correspon al registre
      v_consum := ROUND(((reg.valor_lectura / v_mitja) * 100) - 100, 2);
      -- guardam el registre
      l_proc1 (v_cont) := obj_proc1 (reg.idcontracte, reg.potencia, v_consum);
   END LOOP;

   rsp := 'OK - f_consum_superior_80';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_consum_superior_80', SYSDATE, v_parin, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc1;
EXCEPTION
   -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
   WHEN err_null
   THEN
      rsp := 'ERROR: hi ha camps obligatoris sense informar';

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_consum_superior_80', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_consum_superior_80', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
END f_consum_superior_80;
/

/**************** Llista 10 centraletes que distribueixen ms energia *********************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc2 AS OBJECT (
   direccio        VARCHAR2(100),
   energia_emesa   NUMBER,
   energia_max     NUMBER
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc2 IS TABLE OF obj_proc2;
/

CREATE OR REPLACE FUNCTION f_energia_central_distribucio
   RETURN tab_proc2
IS
/*******************************************************************************************
    NOM:        f_energia_central_distribucio
    PROPSIT:   Funci que retorna una llista amb les 10 centraletes de distribuci que
                distribueixen ms energia
                Valors de la llista:
                - direccio (central_distribucio)
                - Energia_emesa
                - energia_max
    PARMETRES:
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna una llista de, com a molt, 10 registres, amb les centrals de distribuci
            que distribueixen ms energia.
            RSP = 'OK - f_energia_central_distribucio'
        missatge de errors:
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_energia_central_distribucio());
*******************************************************************************************/
   l_proc2    tab_proc2       := tab_proc2 ();
   err_null   EXCEPTION;
   v_cont     NUMBER          := 0;
   rsp        VARCHAR2 (2000);
   err_num    NUMBER;
   err_msg    VARCHAR2 (100);

   CURSOR c_energia_distribuida
   IS
      SELECT *
        FROM (SELECT   cd.direccio, SUM (c.consum_actual) energia_emesa, cd.energia_max
                  FROM comptador c, distribucio_comptador d, central_distribucio cd
                 WHERE d.idcontracte = c.idcontracte AND cd.idcentraldis = d.idcentraldis
              GROUP BY cd.direccio, cd.energia_max
              ORDER BY energia_emesa DESC)
       WHERE ROWNUM <= 10;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   FOR reg IN c_energia_distribuida
   LOOP
      l_proc2.EXTEND;
      v_cont := v_cont + 1;
      -- guardam el registre
      l_proc2 (v_cont) := obj_proc2 (reg.direccio, reg.energia_emesa, reg.energia_max);
   END LOOP;

   rsp := 'OK - f_energia_central_distribucio';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_energia_central_distribucio', SYSDATE, NULL, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc2;
EXCEPTION
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_energia_central_distribucio', SYSDATE, NULL, 'RSP = ' || rsp
                  );

      COMMIT;
END f_energia_central_distribucio;
/

/************ Llista 10 linies de comunicaci amb mes percentatge de crrega **************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc3 AS OBJECT (
   idlinia           NUMBER,
   carrega           NUMBER,
   energia_permesa   NUMBER
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc3 IS TABLE OF obj_proc3;
/

CREATE OR REPLACE FUNCTION f_carrega_linia_comunicacio
   RETURN tab_proc3
IS
/*******************************************************************************************
    NOM:        f_carrega_linia_comunicacio
    PROPSIT:   Funci que retorna una llista amb les 10 lnies ms carregades en relaci
                a la seva capacitat mxima.
                Valors de la llista:
                - idlinia
                - carrega_linia
                - energia_permesa
    PARMETRES:
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna una llista de, com a molt, 10 registres, amb les centrals de distribuci
            que distribueixen ms energia.
            RSP = 'OK - f_carrega_linia_comunicacio'
        missatge de errors:
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_carrega_linia_comunicacio());
*******************************************************************************************/
   l_proc3    tab_proc3       := tab_proc3 ();
   err_null   EXCEPTION;
   v_cont     NUMBER          := 0;
   rsp        VARCHAR2 (2000);
   err_num    NUMBER;
   err_msg    VARCHAR2 (100);

   CURSOR c_carrega_linia
   IS
      SELECT *
        FROM (SELECT   l.idlinia,
                       ROUND ((SUM (c.consum_actual) / l.capacitat_max) * 100, 2) carrega,
                       cp.energia_max - SUM (c.consum_actual) energia_permesa
                  FROM comptador c,
                       distribucio_comptador cd,
                       central_distribucio d,
                       linia_comunicacio l,
                       central_produccio cp
                 WHERE cd.idcontracte = c.idcontracte
                   AND d.idcentraldis = cd.idcentraldis
                   AND l.idlinia = d.idlinia
                   AND cp.idcentralprod = l.idcentralprod
              GROUP BY l.idlinia, l.capacitat_max, cp.energia_max
              ORDER BY carrega DESC)
       WHERE ROWNUM <= 10;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   FOR reg IN c_carrega_linia
   LOOP
      l_proc3.EXTEND;
      v_cont := v_cont + 1;
      -- guardam el registre
      l_proc3 (v_cont) := obj_proc3 (reg.idlinia, reg.carrega, reg.energia_permesa);
   END LOOP;

   rsp := 'OK - f_carrega_linia_comunicacio';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_carrega_linia_comunicacio', SYSDATE, NULL, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc3;
EXCEPTION
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_carrega_linia_comunicacio', SYSDATE, NULL, 'RSP = ' || rsp
                  );

      COMMIT;
END f_carrega_linia_comunicacio;
/

/**************** Llista clients amb contadors en alta disponibilitat *********************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc4 AS OBJECT (
   nif           VARCHAR2 (10),
   idcontracte   NUMBER,
   nommodel      VARCHAR2 (50)
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc4 IS TABLE OF obj_proc4;
/

CREATE OR REPLACE FUNCTION f_clients_compt_disponibilitat
   RETURN tab_proc4
IS
/*******************************************************************************************
    NOM:        f_clients_compt_disponibilitat
    PROPSIT:   Funci que retorna una llista amb els clients que disposen de comptadors
                en alta disponibilitat. Entenem alta disponibilitat que estiguin
                donats d'alta i sense informar data de baixa, el comptador,
                les centrals de distribuci del comptador, les linies de les centrals de
                distribuci i la central de producci.
                Valors de la llista:
                - nif del client
                - idcontracte
                - nommodel de comptador
    PARMETRES:
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna una llista amb els valors de nif, idcontracte i nommodel que compleixen
            els requisits d'alta disponibilitat.
            RSP = 'OK - f_clients_compt_disponibilitat'
        missatge de errors:
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_clients_compt_disponibilitat);
*******************************************************************************************/
   l_proc4    tab_proc4       := tab_proc4 ();
   err_null   EXCEPTION;
   v_exists   NUMBER;
   v_cont     NUMBER          := 0;
   rsp        VARCHAR2 (2000);
   err_num    NUMBER;
   err_msg    VARCHAR2 (100);

   CURSOR c_clients_ad
   IS
      SELECT   c.nif, c.idcontracte, m.nommodel
          FROM comptador c, model_comptador m
         WHERE c.idmodel = m.idmodel AND c.data_baixa IS NULL
      ORDER BY c.idcontracte;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   FOR reg IN c_clients_ad
   LOOP
      -- comprovarem la data de baixa de les centrals de distribuci, lnies de comunicaci
      -- i central de producci, si hi ha al menys un registre donat de baixa, pasarem al
      -- comptador segent de la llista.
      BEGIN
         v_exists := NULL;

         SELECT 1
           INTO v_exists
           FROM distribucio_comptador dc,
                central_distribucio c,
                linia_comunicacio l,
                central_produccio p
          WHERE dc.idcontracte = reg.idcontracte
            AND c.idcentraldis = dc.idcentraldis
            AND l.idlinia = c.idlinia
            AND p.idcentralprod = l.idcentralprod
            AND (c.data_baixa IS NOT NULL OR l.data_baixa IS NOT NULL OR p.data_baixa IS NOT NULL
                )
            AND ROWNUM = 1;
      EXCEPTION
         WHEN NO_DATA_FOUND
         THEN
            -- En cas de que no hi estiguin donades de baixa,
            -- comprovarem les linies de comunicacio conectades
            l_proc4.EXTEND;
            v_cont := v_cont + 1;
            -- guardam el registre
            l_proc4 (v_cont) := obj_proc4 (reg.nif, reg.idcontracte, reg.nommodel);
      END;
   END LOOP;

   rsp := 'OK - f_clients_compt_disponibilitat';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_clients_compt_disponibilitat', SYSDATE, NULL, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc4;
EXCEPTION
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_clients_compt_disponibilitat', SYSDATE, NULL, 'RSP = ' || rsp
                  );

      COMMIT;
END f_clients_compt_disponibilitat;
/

/******************** Llista consum comptadors de central producci ***********************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc5 AS OBJECT (
   idcontracte   NUMBER,
   consum_cont   NUMBER,
   energia_cpd   NUMBER
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc5 IS TABLE OF obj_proc5;
/

CREATE OR REPLACE FUNCTION f_consum_produccio (
   p_idcentralprod   IN   NUMBER,
   p_data_inici      IN   DATE,
   p_data_fi         IN   DATE
)
   RETURN tab_proc5
IS
/*******************************************************************************************
    NOM:        f_consum_produccio
    PROPSIT:   Funci que retorna una llista amb el consum dels comptadors que depenen de
                la central de producci que hem informat com parmetre amb un rang de dates.
                Valors de la llista:
                - idcontracte
                - consum_cont
                - energia_cpd
    PARMETRES:
        IN:     p_idcentralprod: Id de la central de producci.
                p_data_inici: Data d'inici del rang per comprovar el consum
                p_data_fi: Data de fi del rang per comprovar el consum
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna una llista amb els comptadors, consum dels comptadors i energia produda
            RSP = 'OK - f_consum_produccio'
        missatge de errors:
            'ERROR: la data d'inici no pot superior a la data de fi'
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_consum_produccio(1, '01/10/2011', '31/10/2011'));
*******************************************************************************************/
   l_proc5     tab_proc5       := tab_proc5 ();
   err_null    EXCEPTION;
   err_rang    EXCEPTION;
   v_cont      NUMBER          := 0;
   v_limit     NUMBER;
   v_consum    NUMBER;
   v_energia   NUMBER;
   rsp         VARCHAR2 (2000);
   v_parin     VARCHAR2 (300);
   err_num     NUMBER;
   err_msg     VARCHAR2 (100);

   CURSOR c_consum_comptadors (pc_idcentralprod NUMBER, pc_data_inici DATE, pc_data_fi DATE)
   IS
      SELECT   c.idcontracte, SUM (l.valor_lectura) consum
          FROM lectura_comptador l,
               comptador c,
               distribucio_comptador dc,
               central_distribucio d,
               central_produccio p
         WHERE p.idcentralprod = p_idcentralprod
           AND d.idcentralprod = p.idcentralprod
           AND dc.idcentraldis = d.idcentraldis
           AND c.idcontracte = dc.idcontracte
           AND l.idcontracte = c.idcontracte
           AND TRUNC (l.data_inici) >= TRUNC (p_data_inici)
           AND TRUNC (l.data_fi) <= TRUNC (p_data_fi)
      GROUP BY c.idcontracte
      ORDER BY c.idcontracte DESC;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   v_parin :=
         'p_idcentralprod: '
      || p_idcentralprod
      || ', p_data_inici: '
      || p_data_inici
      || ', p_data_fi: '
      || p_data_fi;

   -- Verifica que estan informats els parmetres d'entrada
   IF (TRIM (p_idcentralprod) IS NULL OR TRIM (p_data_inici) IS NULL OR TRIM (p_data_fi) IS NULL)
   THEN
      RAISE err_null;
   END IF;

   -- Comprovam si es correcte l'asignaci del rang.
   IF p_data_inici > p_data_fi
   THEN
      RAISE err_rang;
   END IF;

   -- Agafarem el valor del l'energia produda per la central de produccio en el rang determinat
   SELECT NVL (SUM (energia_mes), 0)
     INTO v_energia
     FROM hist_central_prod
    WHERE idcentralprod = p_idcentralprod
      AND TO_CHAR (data_inici, 'DD/MM/YYYY') >= TO_CHAR (p_data_inici, 'DD/MM/YYYY')
      AND TO_CHAR (data_fi, 'DD/MM/YYYY') <= TO_CHAR (p_data_fi, 'DD/MM/YYYY');

   -- Agafarem la llista del consum dels comptadors
   FOR reg IN c_consum_comptadors (p_idcentralprod, p_data_inici, p_data_fi)
   LOOP
      l_proc5.EXTEND;
      v_cont := v_cont + 1;
      -- Calcularem el % que correspon al registre

      -- guardam el registre
      l_proc5 (v_cont) := obj_proc5 (reg.idcontracte, reg.consum, v_energia);
   END LOOP;

   rsp := 'OK - f_consum_produccio';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_consum_produccio', SYSDATE, v_parin, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc5;
EXCEPTION
   -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
   WHEN err_null
   THEN
      rsp := 'ERROR: hi ha camps obligatoris sense informar';

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_consum_produccio', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
   WHEN err_rang
   THEN
      rsp := 'ERROR: la data d''inici no pot superior a la data de fi';

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_consum_produccio', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_consum_produccio', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
END f_consum_produccio;
/

/******************* Percentatge de lectures en un periode de temps ***********************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc6 AS OBJECT (
   perc_telematica   NUMBER,
   perc_presencial   NUMBER
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc6 IS TABLE OF obj_proc6;
/

CREATE OR REPLACE FUNCTION f_perc_lectura (p_data_inici IN DATE, p_data_fi IN DATE)
   RETURN tab_proc6
IS
/*******************************************************************************************
    NOM:        f_perc_lectura
    PROPSIT:   Funci que retorna una lnia de registre amb els percentatges de lectures
                efectuades de forma presencial i de forma telemtica.
                Valors de la llista:
                - perc_telematica
                - perc_presencial
    PARMETRES:
        IN:     p_data_inici: Data d'inici del rang per comprovar les lectures
                p_data_fi: Data de fi del rang per comprovar les lectures
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna un lnia de registre amb els percentatges corresponents.
            RSP = 'OK - f_perc_lectura'
        missatge de errors:
            'ERROR: la data d'inici no pot superior a la data de fi'
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_perc_lectura('01/03/2012', '31/03/2012'));
*******************************************************************************************/
   l_proc6        tab_proc6       := tab_proc6 ();
   err_null       EXCEPTION;
   err_rang       EXCEPTION;
   v_cont         NUMBER          := 0;
   v_presencial   NUMBER;
   v_telematica   NUMBER;
   v_total        NUMBER;
   v_perc_pres    NUMBER;
   v_perc_tel     NUMBER;
   rsp            VARCHAR2 (2000);
   v_parin        VARCHAR2 (300);
   err_num        NUMBER;
   err_msg        VARCHAR2 (100);

   CURSOR c_lectura_comptadors (
      pc_data_inici   DATE,
      pc_data_fi      DATE,
      pc_modalitat    NUMBER DEFAULT NULL
   )
   IS
      SELECT COUNT (1)
        FROM lectura_comptador
       WHERE idmodalitat = NVL (pc_modalitat, idmodalitat)
         AND data_lectura BETWEEN p_data_inici AND p_data_fi;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   v_parin := 'p_data_inici: ' || p_data_inici || ', p_data_fi: ' || p_data_fi;

   -- Verifica que estan informats els parmetres d'entrada
   IF (TRIM (p_data_inici) IS NULL OR TRIM (p_data_fi) IS NULL)
   THEN
      RAISE err_null;
   END IF;

   -- Comprovam si es correcte l'asignaci del rang.
   IF p_data_inici > p_data_fi
   THEN
      RAISE err_rang;
   END IF;

   -- Agafarem els valors per fer els percentatges

   -- Lectures presencials --
   OPEN c_lectura_comptadors (p_data_inici, p_data_fi, 1);

   FETCH c_lectura_comptadors
    INTO v_telematica;

   CLOSE c_lectura_comptadors;

   -- Lectures telemtiques --
   OPEN c_lectura_comptadors (p_data_inici, p_data_fi, 2);

   FETCH c_lectura_comptadors
    INTO v_presencial;

   CLOSE c_lectura_comptadors;

   -- Lectures totals --
   OPEN c_lectura_comptadors (p_data_inici, p_data_fi);

   FETCH c_lectura_comptadors
    INTO v_total;

   CLOSE c_lectura_comptadors;

   -- Calcularem els percentatges
   v_perc_pres := ROUND ((v_presencial / v_total) * 100, 2);
   v_perc_tel := ROUND ((v_telematica / v_total) * 100, 2);
   -- guardam el registre
   l_proc6.EXTEND;
   v_cont := v_cont + 1;
   l_proc6 (v_cont) := obj_proc6 (v_perc_pres, v_perc_tel);
   rsp := 'OK - f_perc_lectura';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_perc_lectura', SYSDATE, v_parin, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc6;
EXCEPTION
   -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
   WHEN err_null
   THEN
      rsp := 'ERROR: hi ha camps obligatoris sense informar';

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_perc_lectura', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
   WHEN err_rang
   THEN
      rsp := 'ERROR: la data d''inici no pot superior a la data de fi';

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_perc_lectura', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_perc_lectura', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
END f_perc_lectura;
/

/************************** Anys d'antiguitat dels comptadors *****************************/
-- Objecte amb els camps del registre

CREATE OR REPLACE TYPE obj_proc7 AS OBJECT (
   idcontracte   NUMBER,
   data_alta     DATE
);
/

-- Taula que guarda tots els registres del objecte  

CREATE OR REPLACE TYPE tab_proc7 IS TABLE OF obj_proc7;
/

CREATE OR REPLACE FUNCTION f_antiguitat_comptadors (p_anys IN NUMBER)
   RETURN tab_proc7
IS
/*******************************************************************************************
    NOM:        f_antiguitat_comptadors
    PROPSIT:   Funci que retorna una llista amb tots els comptadors que tenen com a data
                d'alta .
                Valors de la llista:
                - idcontracte
                - data_alta
    PARMETRES:
        IN:     p_anys: Nombre d'anys d'antiguitat
        OUT:    RSP
    RETORNA:
        tot ha anat be:
            Retorna una llista amb les comptadors que compleixen la condici.
            RSP = 'OK - f_antiguitat_comptadors'
        missatge de errors:
            'ERROR: hi ha camps obligatoris sense informar'
            'ERROR: Error no tractat'
    EXECUCI:
       SELECT * FROM TABLE(f_antiguitat_comptadors(3));
*******************************************************************************************/
   l_proc7    tab_proc7       := tab_proc7 ();
   err_null   EXCEPTION;
   v_cont     NUMBER          := 0;
   rsp        VARCHAR2 (2000);
   v_parin    VARCHAR2 (300);
   err_num    NUMBER;
   err_msg    VARCHAR2 (100);

   -- Cursor que comprova l'antiguitat del comptador
   -- Agafarem com a referencia la data del sistema, per agafar el valor, prendrem com
   -- antiguitat d'un any tots aquells registres que tinguin l'any complet o ms.
   -- EXEMPLE
   --    data sistema -> '01/05/2012'
   --    data alta    -> '02/06/2011' = menor d'un any = 0 anys
   --    data sistema -> '01/05/2012'
   --    data alta    -> '02/03/2011' = igual o major d'un any = 1 any
   CURSOR c_antiguitat_comptador (pc_anys NUMBER)
   IS
      SELECT   c.idcontracte, c.data_alta
          FROM comptador c
         WHERE pc_anys = (SELECT TRUNC (MONTHS_BETWEEN (TRUNC (SYSDATE), c1.data_alta) / 12)
                                                                                            anys
                            FROM comptador c1
                           WHERE c1.idcontracte = c.idcontracte)
      ORDER BY c.idcontracte;

   PRAGMA AUTONOMOUS_TRANSACTION;
BEGIN
   v_parin := 'pc_anys: ' || p_anys;

   -- Verifica que estan informats els parmetres d'entrada
   IF (TRIM (p_anys) IS NULL)
   THEN
      RAISE err_null;
   END IF;

   -- Agafarem la llista del consum dels comptadors
   FOR reg IN c_antiguitat_comptador (p_anys)
   LOOP
      l_proc7.EXTEND;
      v_cont := v_cont + 1;
      -- Calcularem el % que correspon al registre

      -- guardam el registre
      l_proc7 (v_cont) := obj_proc7 (reg.idcontracte, reg.data_alta);
   END LOOP;

   rsp := 'OK - f_antiguitat_comptadors';

   INSERT INTO logs
               (nomproc, logdate, parin, parout
               )
        VALUES ('f_antiguitat_comptadors', SYSDATE, v_parin, 'RSP = ' || rsp
               );

   COMMIT;
   RETURN l_proc7;
EXCEPTION
   -- Si no se pot fer l'insert, se genera un registre de error a la taula LOGS
   WHEN err_null
   THEN
      rsp := 'ERROR: hi ha camps obligatoris sense informar';

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_antiguitat_comptadors', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
   WHEN OTHERS
   THEN
      err_num := SQLCODE;
      err_msg := SUBSTR (SQLERRM, 1, 200);
      rsp := 'Error no tractat: ' || TO_CHAR (err_num) || ' - ' || err_msg;

      INSERT INTO logs
                  (nomproc, logdate, parin, parout
                  )
           VALUES ('f_antiguitat_comptadors', SYSDATE, v_parin, 'RSP = ' || rsp
                  );

      COMMIT;
END f_antiguitat_comptadors;
/

SPOOL off;
/